"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var typedHtml = require("typed-html");
var resultTable_1 = require("./resultTable");
var layout_1 = require("./layout");
var report_1 = require("stryker-api/report");
function sourceFile(result, sourceFile, mutants, depth, thresholds) {
    return layout_1.layout(result.name, depth, typedHtml.createElement("div", { class: "col-md-12" },
        typedHtml.createElement("div", { class: "row" },
            typedHtml.createElement("div", { class: "col-xs-11" }, resultTable_1.resultTable(result, result.name, thresholds))),
        typedHtml.createElement("div", { class: "row" },
            typedHtml.createElement("div", { class: "col-sm-6" },
                typedHtml.createElement("a", { href: "#", class: "stryker-collapse-expand-all" }, "Expand all")),
            typedHtml.createElement("div", { class: "col-sm-offset-2 col-sm-4" },
                typedHtml.createElement("label", null,
                    typedHtml.createElement("input", { class: "stryker-display-killed", type: "checkbox" }),
                    " Also show killed mutants"))),
        code(sourceFile, mutants)));
}
exports.sourceFile = sourceFile;
function code(sourceFile, mutants) {
    if (sourceFile) {
        return annotateCode(sourceFile, mutants);
    }
    else {
        return typedHtml.createElement("pre", null,
            typedHtml.createElement("code", null, "The source code itself was not reported at the `stryker-html-reporter`. Please report this issue at https://github.com/stryker-mutator/stryker/issues"));
    }
}
function annotateCode(sourceFile, mutants) {
    var currentCursorMutantStatuses = {
        killed: 0,
        survived: 0,
        timeout: 0,
        noCoverage: 0
    };
    var maxIndex = sourceFile.content.length - 1;
    var numberedMutants = _
        .sortBy(mutants, function (m) { return m.range[0] * 10000 + m.range[1] * -1; })
        .map(function (mutant, index) { return ({ mutant: mutant, index: index }); });
    var adjustCurrentMutantResult = function (valueToAdd) { return function (numberedMutant) {
        switch (numberedMutant.mutant.status) {
            case report_1.MutantStatus.Killed:
                currentCursorMutantStatuses.killed += valueToAdd;
                break;
            case report_1.MutantStatus.Survived:
                currentCursorMutantStatuses.survived += valueToAdd;
                break;
            case report_1.MutantStatus.TimedOut:
                currentCursorMutantStatuses.timeout += valueToAdd;
                break;
            case report_1.MutantStatus.NoCoverage:
                currentCursorMutantStatuses.noCoverage += valueToAdd;
                break;
        }
    }; };
    var determineBackground = function () {
        if (currentCursorMutantStatuses.survived > 0) {
            return getContextClassForStatus(report_1.MutantStatus.Survived);
        }
        else if (currentCursorMutantStatuses.noCoverage > 0) {
            return getContextClassForStatus(report_1.MutantStatus.NoCoverage);
        }
        else if (currentCursorMutantStatuses.timeout > 0) {
            return getContextClassForStatus(report_1.MutantStatus.TimedOut);
        }
        else if (currentCursorMutantStatuses.killed > 0) {
            return getContextClassForStatus(report_1.MutantStatus.Killed);
        }
        return null;
    };
    var annotate = function (char, index) {
        var mutantsStarting = numberedMutants.filter(function (m) { return m.mutant.range[0] === index; });
        var mutantsEnding = numberedMutants.filter(function (m) { return m.mutant.range[1] === index; });
        mutantsStarting.forEach(adjustCurrentMutantResult(1));
        mutantsEnding.forEach(adjustCurrentMutantResult(-1));
        var backgroundColorAnnotation = mutantsStarting.length || mutantsEnding.length || index === 0 ? "<span class=\"bg-" + determineBackground() + "\">" : '';
        var backgroundColorEndAnnotation = ((mutantsStarting.length || mutantsEnding.length) && index > 0) || index === maxIndex ? '</span>' : '';
        try {
            var mutantsAnnotations = mutantsStarting.map(function (m) {
                return typedHtml.createElement("a", { href: "#", class: "stryker-mutant-button", "data-mutant-status-annotation": getContextClassForStatus(m.mutant.status), "data-mutant": m.index },
                    typedHtml.createElement("span", { class: "label label-" + getContextClassForStatus(m.mutant.status) }, m.index))
                    + typedHtml.createElement("span", { class: "label label-info stryker-mutant-replacement", hidden: "hidden", "data-mutant": m.index }, escapeHtml(m.mutant.replacement));
            });
            var originalCodeStartAnnotations = mutantsStarting.map(function (m) { return "<span class=\"stryker-original-code\" data-mutant=\"" + m.index + "\">"; });
            var originalCodeEndAnnotations = mutantsEnding.map(function (m) { return '</span>'; });
            return "" + backgroundColorEndAnnotation + originalCodeEndAnnotations.join('') + mutantsAnnotations.join('') + originalCodeStartAnnotations.join('') + backgroundColorAnnotation + escapeHtml(char);
        }
        catch (err) {
            console.log(err);
        }
    };
    return typedHtml.createElement("pre", null,
        typedHtml.createElement("code", { class: "lang-javascript" }, mapString(sourceFile.content, annotate).join('')));
}
function escapeHtml(unsafe) {
    return unsafe
        .replace(/&/g, '&amp;')
        .replace(/</g, '&lt;')
        .replace(/>/g, '&gt;')
        .replace(/"/g, '&quot;')
        .replace(/'/g, '&#039;');
}
function getContextClassForStatus(status) {
    switch (status) {
        case report_1.MutantStatus.Killed:
            return 'success';
        case report_1.MutantStatus.NoCoverage:
        case report_1.MutantStatus.Survived:
            return 'danger';
        default:
            return 'warning';
    }
}
/**
 * A `map` function, as in [1, 2].map(i => i+1), but for a string
 */
function mapString(source, fn) {
    var results = [];
    for (var i = 0; i < source.length; i++) {
        results.push(fn(source[i], i));
    }
    return results;
}
//# sourceMappingURL=sourceFile.js.map